#!/bin/bash
# Usage:

function validate 
{
	if ! grep "FROMIB COMPLETE      8 SWITCHTABLE" $1.STARSxrefs > /dev/null; then
	    echo "Failed to find COMPLETE 8 SWITCHTABLE xref annotation"
            echo "COMPLETE SWITCHTABLE annotations: "
            grep "SWITCHTABLE" $1.STARSxrefs 2>&1 | tee cat
		exit 3
	fi
	if ! grep "FROMIB COMPLETE      1 RETURNTARGET" $1.STARSxrefs > /dev/null; then
		echo "Failed to find any COMPLETE RETURNTARGET xref annotations"
		exit 2
    	fi
	if ! tail $1.infoannot | grep "SUCCESS ANALYSISCOMPLETED" > /dev/null; then
		echo "Failed to find SUCCESS ANALYSISCOMPLETED annotation"
		exit 1
	fi

	echo "STARS success"
	return 0
}

function test() 
{
    . $SMPSA_HOME/SMP-fullanalyze64-ida7.sh $1
    validate $1
}

# Pass in optimization level string, e.g. O0, Og, or O3
function build() 
{
    rm -f starstest2_$1.exe
    gcc -m64 -$1 -o starstest2_$1.exe $SMPSA_HOME/tests/starstest2.c
}

# Pass in optimization level string, e.g. O0, Og, or O3
function build_and_test()
{
    build $1
    test starstest2_$1.exe
}

function main()
{
	set -e
	set -x

	cd $CICD_MODULE_WORK_DIR/cicd_stars_umbrella
        export IDAROOT=$CICD_MODULE_WORK_DIR/idapro71
        export IDASDK=$CICD_MODULE_WORK_DIR/idapro71_sdk

	source set_env_vars
	mkdir -p /tmp/stars_test
	cd /tmp/stars_test

	# Build and test each of the optimization levels -O0, -Og, and -O3.
	if build_and_test O0 ; then
	    if build_and_test Og ; then
		build_and_test O3
	    fi
	fi

	return 0

}

main "$@"
