#include <memory>

#include "interfaces/abstract/STARSInterface.h"
#include "interfaces/irdb/STARSSegment.h"
#include "interfaces/irdb/STARSFunction.h"
#include "interfaces/irdb/STARSInstruction.h"
#include "interfaces/irdb/STARSSSAInstruction.h"
#include "interfaces/irdb/STARSFunction.h"
#include <elfio/elfio.hpp>

#include <irdb-core>
#include <irdb-util>
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <pqxx/pqxx>

using namespace std;

unique_ptr<STARS_Instruction_t> STARS_IRDB_Interface_t::CreateInst(STARS_InstructionID_t InstID)
{
	// check for pseudo-instructions
	if (STARS_IsSSAMarkerPseudoID(InstID.GetIDWithinFile()))
		return unique_ptr<STARS_Instruction_t>{(STARS_Instruction_t *) new STARS_IRDB_Instruction_t(InstID)};

	// already created, just return what we need.
	STARS_Instruction_t* insn = nullptr;

	if (InstID.HasCorrespondingInstructionIR())
	{
		return unique_ptr<STARS_Instruction_t>{(STARS_Instruction_t*)InstID.GetInstruction()};
	}
	else
	{
		// Look it up in the private map, create STARS_IRDB_Instruction_t with it.
		return unique_ptr<STARS_Instruction_t>{(STARS_Instruction_t*) new STARS_IRDB_Instruction_t(this->instr_id_to_irdb_insn_map[(IRDB_SDK::DatabaseID_t) InstID.GetIDWithinFile()])};
	}

};


bool STARS_IRDB_Interface_t::STARS_generate_disasm_line(STARS_ea_t addr, char *buf, std::size_t bufsize, int flags)
{
	STARS_InstructionID_t InstID(addr);

	IRDB_SDK::DatabaseID_t IRDBInstID = (IRDB_SDK::DatabaseID_t) InstID.GetIDWithinFile();
	IRDB_SDK::Instruction_t* irdb_insn = this->instr_id_to_irdb_insn_map[IRDBInstID];

	strncpy(buf, irdb_insn->getDisassembly().c_str(), bufsize);
	return true;
}

bool STARS_IRDB_Interface_t::STARS_getenv(const char *varname, char varvalue[STARS_MAXSTR]) const {
	char *EnvValuePtr = getenv(varname);
	if (nullptr != EnvValuePtr) {
		(void)strncpy(&varvalue[0], EnvValuePtr, STARS_MAXSTR-1);
	}
	return (nullptr != EnvValuePtr); 
};


bool STARS_IRDB_Interface_t::IsInstJumpTarget(STARS_InstructionID_t InstID)
{
#if 0
    const STARS_IRDB_Instruction_t* interface_insn=dynamic_cast<const STARS_IRDB_Instruction_t*>(InstID.GetInstruction());
    assert(interface_insn);
	IRDB_SDK::Instruction_t* insn=(IRDB_SDK::Instruction_t*)interface_insn->GetIRDBInstruction();
#else
	IRDB_SDK::DatabaseID_t IRDBInstID = (IRDB_SDK::DatabaseID_t) InstID.GetIDWithinFile();
	IRDB_SDK::Instruction_t* insn = this->instr_id_to_irdb_insn_map[IRDBInstID];
#endif

    // return true if
    //      jumped to indirectly (jmp instruction)
    //      called indirectly (call instruction)
    // Using the IBT provs guarantees we don't count jumped to by a ret 
    // (which includes push-rets), even with unpinned IBTs 
    if (IBT_provs[insn].hasIndirectJump() || IBT_provs[insn].hasIndirectCall())
	{
		return true;
	}
        
    //      jumped to directly (jmp or jcc instruction)
    //      called directly (call instruction)

    for (auto pred : instruction_preds[insn])
    {
        if (pred->getTarget() == insn)
            return true;
    } 

    return false;


        //      jumped to via a return doesn't count.
        // be careful and think about this as you impl it.
        // not sure how to do this, because we don't know if a return will go there.
        // typically what is meant by a "return goes there" is that a call
        // instruction preceeds it.  Since call instructions don't (typically) mark
        // the return point as an IBT, we should be OK.

        // aka -- not (is only-fallthroughed to)
        assert(0); return 0;
}

STARS_InstructionID_t  STARS_IRDB_Interface_t::FindFirstCallTarget(STARS_InstructionID_t CallInstID)
#if 0
                // direct call: if no target -- return STARS_BADADDR
                // direct call: if target, return target
                // indirect call: if any targets exist return one.
                // indirect call: if no known targets, return  STARS_BADADDR ?
                assert(0); return 0;
#endif
{
#if 0
	const STARS_Instruction_t* interface_insn = CallInstID.GetInstruction();
	assert(interface_insn);
	const STARS_IRDB_Instruction_t* irdb_interface_insn = dynamic_cast<const STARS_IRDB_Instruction_t*>(interface_insn);
	assert(irdb_interface_insn);
	const IRDB_SDK::Instruction_t* irdb_insn=irdb_interface_insn->GetIRDBInstruction();
#else
	IRDB_SDK::DatabaseID_t InstID = (IRDB_SDK::DatabaseID_t) CallInstID.GetIDWithinFile();
	IRDB_SDK::Instruction_t* irdb_insn = this->instr_id_to_irdb_insn_map[InstID];
#endif

	if(irdb_insn->getTarget())
		return STARS_InstructionID_t(irdb_insn->getTarget()->getBaseID());

	// can't be a direct call, for now assume we don't know where it goes.
	return STARS_BADADDR;
}



bool operator<(const STARS_IRDB_Interface_t::AddressIDPlaceholder_t& lhs, const STARS_IRDB_Interface_t::AddressIDPlaceholder_t& rhs)
{
        return std::tie(lhs.fileID,lhs.virtual_offset) < std::tie(rhs.fileID,rhs.virtual_offset);
}

