
#include "interfaces/irdb/STARSOp.h"
#include "interfaces/STARSTypes.h"
#include "interfaces/SMPDBInterface.h"
#include "interfaces/irdb/STARSProgram.h"
#include "interfaces/irdb/STARSInstruction.h"
#include "interfaces/abstract/STARSInterface.h"

#include <irdb-core>
#include <iostream>

using namespace std;
using namespace IRDB_SDK;

static int log2int(unsigned int index)
{
	assert(index>=0);
	int targetlevel = 0;
	while (index >>= 1) ++targetlevel;
	return  targetlevel;
}

#if 0

static int log2int_or_err(unsigned int index)
{
	if(index==0)
		return -1;
	return log2int(index);
}
#endif


STARS_IRDB_op_t::STARS_IRDB_op_t(const DecodedInstruction_t& d, int indx, const shared_ptr<DecodedOperand_t>& the_arg_p, const int length)
{
	auto &the_arg=*the_arg_p;
	Init(); // make void
	assert(length>=0 && length<=16); // x86-insns are 1-16 bytes, but we allow 0 in case you're lazy.

	visible=1;
	if(the_arg.isRegister() && the_arg.isGeneralPurposeRegister())
	{
		OpType=op_Reg;

		     if(the_arg.getString()=="al" )  { operand.reg.RegNum=STARS_x86_R_al ; byteWidth=1; }
		else if(the_arg.getString()=="cl" )  { operand.reg.RegNum=STARS_x86_R_cl ; byteWidth=1; }
		else if(the_arg.getString()=="dl" )  { operand.reg.RegNum=STARS_x86_R_dl ; byteWidth=1; }
		else if(the_arg.getString()=="bl" )  { operand.reg.RegNum=STARS_x86_R_bl ; byteWidth=1; }
		else if(the_arg.getString()=="ah" )  { operand.reg.RegNum=STARS_x86_R_ah ; byteWidth=1; }
		else if(the_arg.getString()=="ch" )  { operand.reg.RegNum=STARS_x86_R_ch ; byteWidth=1; }
		else if(the_arg.getString()=="dh" )  { operand.reg.RegNum=STARS_x86_R_dh ; byteWidth=1; }
		else if(the_arg.getString()=="bh" )  { operand.reg.RegNum=STARS_x86_R_bh ; byteWidth=1; }
		else if(the_arg.getString()=="spl")  { operand.reg.RegNum=STARS_x86_R_spl; byteWidth=1; }
		else if(the_arg.getString()=="spl")  { operand.reg.RegNum=STARS_x86_R_spl; byteWidth=1; }
		else if(the_arg.getString()=="bpl")  { operand.reg.RegNum=STARS_x86_R_bpl; byteWidth=1; }
		else if(the_arg.getString()=="sil")  { operand.reg.RegNum=STARS_x86_R_sil; byteWidth=1; }
		else if(the_arg.getString()=="dil")  { operand.reg.RegNum=STARS_x86_R_dil; byteWidth=1; }
		else 
		{
			// set the operand.reg.RegNum appropriately.
			const auto bea_regno=the_arg.getRegNumber(); // log2int_or_err(the_arg.ArgType&0xFFFF);
			operand.reg.RegNum=(STARS_RegNo)(STARS_x86_R_ax+bea_regno);		// regno may be -1, but ax-1==none, which is right.

			// if the first character is an r, it's a 64-bit reg.
			// e.g. rax, rsp, rdi; UNLESS there is a suffix
			// if (the_arg.ArgMnemonic[0] == 'r') 
			if (the_arg.getString()[0] == 'r') 
			{
				byteWidth = 8;
				size_t ArgNameLen = the_arg.getString().length() /*strlen(the_arg.ArgMnemonic)*/;
				char Suffix = (char) tolower(the_arg.getString()[ArgNameLen - 1]);
				if ('b' == Suffix)
					byteWidth = 1;
				else if ('w' == Suffix)
					byteWidth = 2;
				else if ('d' == Suffix)
					byteWidth = 4;
			}

			// otherwise, if the first character is an e, and the 3rd char is an 'x', 'p', or 'i'  it's a 32-bit reg. 
			// e.g., eax, esp, edi
			// caution: es
			// else if(the_arg.ArgMnemonic[0]=='e' && (the_arg.ArgMnemonic[2]=='x' || the_arg.ArgMnemonic[2]=='i' || the_arg.ArgMnemonic[2]=='p'))
			else if(the_arg.getString()[0]=='e' && (the_arg.getString()[2]=='x' || the_arg.getString()[2]=='i' || the_arg.getString()[2]=='p'))
				byteWidth=4;
			// e.g	ax, sp, bp, di
			// else if(the_arg.ArgMnemonic[1]=='x' || the_arg.ArgMnemonic[1]=='i' || the_arg.ArgMnemonic[1]=='p')
			else if(the_arg.getString()[1]=='x' || the_arg.getString()[1]=='i' || the_arg.getString()[1]=='p')
				byteWidth=2;
			else 
				byteWidth=1;
		}
	}
	/*case REGISTER_TYPE + MMX_REG : */
	else if (the_arg.isRegister() && the_arg.isMmxRegister()) 
	{
		const auto bea_regno=the_arg.getRegNumber(); // log2int_or_err(the_arg.ArgType&0xFFFF);
		byteWidth=8;
#if 1	// Use specialized register types.
		OpType = op_MMXReg;
#else		// Using op_Reg for all regs, distinguishing them by the regno only.
		OpType = op_Reg;
#endif
		assert(bea_regno != (decltype(bea_regno))-1);
		operand.reg.RegNum=(STARS_RegNo)(STARS_x86_R_mm0+bea_regno);
	}
	// case REGISTER_TYPE + FPU_REG :
	else if (the_arg.isRegister() && the_arg.isFpuRegister()) 
	{
		const auto bea_regno=the_arg.getRegNumber(); // log2int_or_err(the_arg.ArgType&0xFFFF);
		byteWidth=8;
#if 1	// Use specialized register types.
		OpType = op_FPReg;
#else		// Using op_Reg for all regs, distinguishing them by the regno only.
		OpType = op_Reg;
#endif
		assert(bea_regno != (decltype(bea_regno))-1);
		operand.reg.RegNum=(STARS_RegNo)(STARS_x86_R_st0+bea_regno);
	}
	// case REGISTER_TYPE + SSE_REG :
	// case REGISTER_TYPE + AVX_REG :
	else if (the_arg.isRegister() && (the_arg.isSseRegister() || the_arg.isAvxRegister()) )
	{
		const auto bea_regno=the_arg.getRegNumber(); // log2int_or_err(the_arg.ArgType&0xFFFF);
		//if (the_arg.ArgMnemonic[0] == 'x')
		if (the_arg.getString()[0] == 'x')
		{
			byteWidth=8;
#if 1	// Use specialized register types.
			OpType=op_XMMReg;
#else		// Using op_Reg for all regs, distinguishing them by the regno only.
			OpType = op_Reg;
#endif
			assert(bea_regno!=(decltype(bea_regno))-1);
			if (16 > bea_regno)
				operand.reg.RegNum=(STARS_RegNo)(STARS_x86_R_xmm0 + bea_regno);
			else
				operand.reg.RegNum = (STARS_RegNo)(STARS_x86_R_xmm16 - 16 + bea_regno);
		}
		else if(the_arg.getString()[0]=='y')
		{
			byteWidth=16;
#if 1	// Use specialized register types.
			OpType = op_YMMReg;
#else		// Using op_Reg for all regs, distinguishing them by the regno only.
			OpType = op_Reg;
#endif
			assert(bea_regno != (decltype(bea_regno))-1);
			if (16 > bea_regno)
				operand.reg.RegNum = (STARS_RegNo)(STARS_x86_R_ymm0 + bea_regno);
			else 
				operand.reg.RegNum = (STARS_RegNo)(STARS_x86_R_ymm16 - 16 + bea_regno);
		}
		else
			assert(0);
	}
	// case REGISTER_TYPE + SPECIAL_REG :
	else if (the_arg.isRegister() && the_arg.isSpecialRegister()) 
	{
		const auto bea_regno=the_arg.getRegNumber(); // log2int_or_err(the_arg.ArgType&0xFFFF);
/*
 * this isn't even close to right.  should be cr0, dr0, etc., but stars doesn't have that... just use eflags and pray?
*/
		OpType = op_Reg;
		operand.reg.RegNum = (STARS_x86_R_efl);


	}
	// case REGISTER_TYPE + SEGMENT_REG :
	else if (the_arg.isRegister() && the_arg.isSegmentRegister()) 
	{
		const auto bea_regno=the_arg.getRegNumber(); // log2int_or_err(the_arg.ArgType&0xFFFF);
		OpType=op_SegReg;
		assert(bea_regno!=(decltype(bea_regno))-1);
		operand.reg.RegNum=(STARS_RegNo)(STARS_x86_R_es+bea_regno);
	}
	else if (the_arg.isMemory())
	{
		// extract fields
		operand.mem.base  = (STARS_RegNo)(the_arg.hasBaseRegister()  ?  the_arg.getBaseRegister() : STARS_x86_R_none); 
		operand.mem.index = (STARS_RegNo)(the_arg.hasIndexRegister() ?  the_arg.getIndexRegister(): STARS_x86_R_none); 
		operand.mem.scale = the_arg.getScaleValue(); 

		// set things ida-pro style for pcrel operands.
		if( the_arg.isPcrel() ) 
		{
			assert(length!=0);// we need the length in this case.
			operand.mem.disp=the_arg.getMemoryDisplacement() + length; 
			// Imitate IDA Pro: IP-relative static memop is relative to CS segment register.
			this->SegReg = STARS_x86_R_cs;
		}
		else
		{
			operand.mem.disp = the_arg.getMemoryDisplacement(); 
		}


		// if there's no base reg, and an index reg with scale==1,
		// ida represents this oddly as a base with no sib.
		if(operand.mem.base ==  STARS_x86_R_none && operand.mem.scale==1 && operand.mem.index!=STARS_x86_R_none)
		{
			// change representation.
			operand.mem.base=operand.mem.index;
			operand.mem.index=STARS_x86_R_none;
			operand.mem.scale=0;
		}

		// if the scale >0, we have a sib byte.
		if(operand.mem.scale > 0)
		{
			operand.mem.hasSIB=1;
			operand.mem.scale=log2int(operand.mem.scale);	
		}
		// if there's both a base and an index, then there's a sib byte.
		if(operand.mem.base != STARS_x86_R_none && operand.mem.index != STARS_x86_R_none)
			operand.mem.hasSIB=1;


		byteWidth=the_arg.getArgumentSizeInBytes(); 
		OpType=op_Mem;
	}
	// case CONSTANT_TYPE + RELATIVE_ :
	else if (the_arg.isConstant() && the_arg.isPcrel())
	{
		OpType=op_Imm;
		operand.imm.imm=d.getImmediate(); // d.Instruction.Immediat;
		operand.imm.pc_rel=true;
		byteWidth=the_arg.getArgumentSizeInBytes(); // ArgSize/8;
	}
	// case CONSTANT_TYPE + ABSOLUTE_ :
	else if (the_arg.isConstant() && !the_arg.isPcrel())
	{
		OpType=op_Imm;
		switch(the_arg.getArgumentSizeInBytes()*8)
		{
			case 8:
				operand.imm.imm=(signed int)(signed char)the_arg.getConstant(); // d.Instruction.Immediat;
				break;
			case 16:
				operand.imm.imm=(signed int)(signed short)the_arg.getConstant(); // d.Instruction.Immediat;
				break;
			case 32:
			default:
				operand.imm.imm=the_arg.getConstant(); // d.Instruction.Immediat;
				break;
	
		}
		operand.imm.pc_rel=false;
		byteWidth=the_arg.getArgumentSizeInBytes(); // ArgSize/8;
	}

	// no seg reg
	if(!the_arg.hasSegmentRegister())
		SegReg=STARS_x86_R_none;  // map to no reg.
	// case ESReg:	          
	else if(the_arg.getSegmentRegister()==0)
		SegReg=STARS_x86_R_es;    // STARS_x86_R_es reg.
	//case CSReg 
	else if(the_arg.getSegmentRegister()==1)
		SegReg=STARS_x86_R_cs;
	//case SSReg
	else if(the_arg.getSegmentRegister()==2)
		SegReg=STARS_x86_R_ss;
	// case DSReg
	else if(the_arg.getSegmentRegister()==3)
		SegReg=STARS_x86_R_ds;
	// case FSReg
	else if(the_arg.getSegmentRegister()==4)
		SegReg=STARS_x86_R_fs;
	//case GSReg
	else if(the_arg.getSegmentRegister()==5)
		SegReg=STARS_x86_R_gs;
	else
		assert(0);


}

void STARS_IRDB_op_t::Init(void)
{
	this->SegReg = STARS_x86_R_none;
	this->OpType = op_Void;

	// init the operand to 0, because the it should have no semantically valid meaning in any sense anyhow if OpType==Void.
	memset(&this->operand, 0, sizeof(this->operand));

	this->visible = 0;
	if (nullptr != global_STARS_program)	// global_STARS_program not available for early static initialization
		this->byteWidth = global_STARS_program->GetSTARS_ISA_Bytewidth();
	else
		this->byteWidth = 8;

}

bool STARS_IRDB_op_t::operator<(const STARS_op_t &rOp_param) const
{
	const STARS_IRDB_op_t* p=dynamic_cast<const STARS_IRDB_op_t*>(&rOp_param);
	assert(p);
	const STARS_IRDB_op_t& rOp=*p;

	if(GetOpType() != rOp.GetOpType())
		return GetOpType() < rOp.GetOpType();

	switch(GetOpType())
	{
		case op_Mem: 
			if ( operand.mem.base != rOp.operand.mem.base)
				return ( operand.mem.base < rOp.operand.mem.base);
			if ( operand.mem.index != rOp.operand.mem.index)
				return ( operand.mem.index < rOp.operand.mem.index);
			if ( operand.mem.scale != rOp.operand.mem.scale)
				return ( operand.mem.scale < rOp.operand.mem.scale);
			if ( operand.mem.disp != rOp.operand.mem.disp)
				return ( operand.mem.disp < rOp.operand.mem.disp);
			return false; // equal
		case op_Imm: 
			return operand.imm.imm < rOp.operand.imm.imm;
		case op_CrReg: 
		case op_DrReg: 
		case op_Eflags: 
		case op_MMXReg: 
		case op_XMMReg: 
		case op_YMMReg: 
		case op_FPReg: 
		case op_Reg: 
		case op_SegReg:
			return operand.reg.RegNum < rOp.operand.reg.RegNum;

		case op_Void: 
		case op_Mxcsr: 
			return false; // equal

		case op_Addr:
			return operand.addr.addr < rOp.operand.addr.addr;

		default:
			assert(0);
			
	}
}

char STARS_IRDB_op_t::GetSIB(void) const
{
	char ret=0;
	ret|=((int)operand.mem.base&0x7)<<0;
	ret|=((int)operand.mem.index&0x7)<<3;
	ret|=(((int)operand.mem.scale)&0x3)<<6;
	return ret;
} // Get x86 SIB byte (dense encoding of base reg, index reg, and scale factor)


void STARS_IRDB_op_t::SetSIB(char value)
{
	assert(OpType==op_Mem);
	operand.mem.hasSIB=true;

#pragma GCC diagnostic ignored "-Wstrict-aliasing"
	// update the base register.  Odd casting to avoid compiler error bcause base is a STARS_RegNo
	(*(int*)&operand.mem.base)&=~0x7;
	(*(int*)&operand.mem.base)|=(value&0x7);

	// Set scale.  Odd casting to avoid compiler error bcause base is a STARS_RegNo
	(*(int*)&operand.mem.index)&=~0x7;
	(*(int*)&operand.mem.index)|=(value>>3)&0x7;
#pragma GCC diagnostic pop

	// set scale
	operand.mem.scale= (STARS_RegNo)((value>>6)&0x3);
	if(operand.mem.index==STARS_x86_R_sp)
		operand.mem.scale=(STARS_RegNo)0;

	assert(value==GetSIB());
}

void STARS_IRDB_op_t::Dump()
{

	cout<<"Dumping operand at "<<hex<<this<<endl;
	cout<<"Byte width:"<<dec<<byteWidth<<endl;
	if(visible)
		cout<<"Operand is visible"<<endl;
	else 
		cout<<"Operand is NOT visible"<<endl;

	switch(OpType)
	{

                case op_Reg:
			cout<<"Type: Reg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_Mem:
			cout<<"Type: Mem["<<MDGetSIBBaseReg()<<"+"<<MDGetSIBIndexReg()<<"*"
			    <<GetSIBScaleFactor()<<"+"<<hex<<GetAddr()<<"]"<<endl;
			break;
		case op_Imm:
			cout<<"Type: Imm:"<<hex<<GetAddr()<<endl;
			break;
		case op_Void:
			cout<<"Type: Void"<<endl;
			break;
		case op_CrReg:
			cout<<"Type: CrReg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_DrReg:
			cout<<"Type: DrReg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_Eflags:
			cout<<"Type: Eflags"<<endl;
			break;
		case op_MMXReg:
			cout<<"Type: MmxReg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_XMMReg:
			cout<<"Type: XmmReg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_YMMReg:
			cout<<"Type: YmmReg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_FPReg:
			cout<<"Type: FpReg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_Mxcsr:
			cout<<"Type: Mxcsr"<<endl;
			break;
		case op_SegReg:
			cout<<"Type: SegReg, regno="<<dec<<GetReg()<<endl;
			break;
		case op_Addr:
			cout<<"Type: Addr:"<<hex<<GetAddr()<<endl;
			break;

		
	}
	
}


uint16_t STARS_IRDB_op_t::GetByteWidth(void) const
{
	if (OpType == op_Void) {
		if (nullptr != global_STARS_program)	// global_STARS_program not available for early static initialization
			return global_STARS_program->GetSTARS_ISA_Bytewidth();
		else
			return 8;
	}
	else
		return byteWidth;
}

